@@ -1,6 +1,8 @@
 // CRATES
 use crate::esc;
-use crate::utils::{catch_random, error, format_num, format_url, param, redirect, rewrite_urls, setting, template, val, Post, Preferences, Subreddit};
+use crate::utils::{
+	catch_random, error, filter_posts, format_num, format_url, get_filters, param, redirect, rewrite_urls, setting, template, val, Post, Preferences, Subreddit,
+};
 use crate::{client::json, server::ResponseExt, RequestExt};
 use askama::Template;
 use cookie::Cookie;
@@ -17,6 +19,11 @@ struct SubredditTemplate {
 	ends: (String, String),
 	prefs: Preferences,
 	url: String,
+	/// Whether the subreddit itself is filtered.
+	is_filtered: bool,
+	/// Whether all fetched posts are filtered (to differentiate between no posts fetched in the first place,
+	/// and all fetched posts being filtered).
+	all_posts_filtered: bool,
 }
 
 #[derive(Template)]
@@ -26,6 +33,7 @@ struct WikiTemplate {
 	wiki: String,
 	page: String,
 	prefs: Preferences,
+	url: String,
 }
 
 #[derive(Template)]
@@ -47,7 +55,7 @@ pub async fn community(req: Request<Body>) -> Result<Response<Body>, String> {
 	let post_sort = req.cookie("post_sort").map_or_else(|| "hot".to_string(), |c| c.value().to_string());
 	let sort = req.param("sort").unwrap_or_else(|| req.param("id").unwrap_or(post_sort));
 
-	let sub = req.param("sub").unwrap_or(if front_page == "default" || front_page.is_empty() {
+	let sub_name = req.param("sub").unwrap_or(if front_page == "default" || front_page.is_empty() {
 		if subscribed.is_empty() {
 			"popular".to_string()
 		} else {
@@ -56,59 +64,77 @@ pub async fn community(req: Request<Body>) -> Result<Response<Body>, String> {
 	} else {
 		front_page.clone()
 	});
-	let quarantined = can_access_quarantine(&req, &sub) || root;
+	let quarantined = can_access_quarantine(&req, &sub_name) || root;
 
 	// Handle random subreddits
-	if let Ok(random) = catch_random(&sub, "").await {
+	if let Ok(random) = catch_random(&sub_name, "").await {
 		return Ok(random);
 	}
 
-	if req.param("sub").is_some() && sub.starts_with("u_") {
-		return Ok(redirect(["/user/", &sub[2..]].concat()));
+	if req.param("sub").is_some() && sub_name.starts_with("u_") {
+		return Ok(redirect(["/user/", &sub_name[2..]].concat()));
 	}
 
-	let path = format!("/r/{}/{}.json?{}&raw_json=1", sub, sort, req.uri().query().unwrap_or_default());
+	// Request subreddit metadata
+	let sub = if !sub_name.contains('+') && sub_name != subscribed && sub_name != "popular" && sub_name != "all" {
+		// Regular subreddit
+		subreddit(&sub_name, quarantined).await.unwrap_or_default()
+	} else if sub_name == subscribed {
+		// Subscription feed
+		if req.uri().path().starts_with("/r/") {
+			subreddit(&sub_name, quarantined).await.unwrap_or_default()
+		} else {
+			Subreddit::default()
+		}
+	} else if sub_name.contains('+') {
+		// Multireddit
+		Subreddit {
+			name: sub_name.clone(),
+			..Subreddit::default()
+		}
+	} else {
+		Subreddit::default()
+	};
 
-	match Post::fetch(&path, String::new(), quarantined).await {
-		Ok((posts, after)) => {
-			// If you can get subreddit posts, also request subreddit metadata
-			let sub = if !sub.contains('+') && sub != subscribed && sub != "popular" && sub != "all" {
-				// Regular subreddit
-				subreddit(&sub, quarantined).await.unwrap_or_default()
-			} else if sub == subscribed {
-				// Subscription feed
-				if req.uri().path().starts_with("/r/") {
-					subreddit(&sub, quarantined).await.unwrap_or_default()
-				} else {
-					Subreddit::default()
-				}
-			} else if sub.contains('+') {
-				// Multireddit
-				Subreddit {
-					name: sub,
-					..Subreddit::default()
-				}
-			} else {
-				Subreddit::default()
-			};
+	let path = format!("/r/{}/{}.json?{}&raw_json=1", sub_name.clone(), sort, req.uri().query().unwrap_or_default());
+	let url = String::from(req.uri().path_and_query().map_or("", |val| val.as_str()));
+	let filters = get_filters(&req);
 
-			let url = String::from(req.uri().path_and_query().map_or("", |val| val.as_str()));
+	// If all requested subs are filtered, we don't need to fetch posts.
+	if sub_name.split('+').all(|s| filters.contains(s)) {
+		template(SubredditTemplate {
+			sub,
+			posts: Vec::new(),
+			sort: (sort, param(&path, "t").unwrap_or_default()),
+			ends: (param(&path, "after").unwrap_or_default(), "".to_string()),
+			prefs: Preferences::new(req),
+			url,
+			is_filtered: true,
+			all_posts_filtered: false,
+		})
+	} else {
+		match Post::fetch(&path, quarantined).await {
+			Ok((mut posts, after)) => {
+				let all_posts_filtered = filter_posts(&mut posts, &filters);
 
-			template(SubredditTemplate {
-				sub,
-				posts,
-				sort: (sort, param(&path, "t").unwrap_or_default()),
-				ends: (param(&path, "after").unwrap_or_default(), after),
-				prefs: Preferences::new(req),
-				url,
-			})
+				template(SubredditTemplate {
+					sub,
+					posts,
+					sort: (sort, param(&path, "t").unwrap_or_default()),
+					ends: (param(&path, "after").unwrap_or_default(), after),
+					prefs: Preferences::new(req),
+					url,
+					is_filtered: false,
+					all_posts_filtered,
+				})
+			}
+			Err(msg) => match msg.as_str() {
+				"quarantined" => quarantine(req, sub_name),
+				"private" => error(req, format!("r/{} is a private community", sub_name)).await,
+				"banned" => error(req, format!("r/{} has been banned from Reddit", sub_name)).await,
+				_ => error(req, msg).await,
+			},
 		}
-		Err(msg) => match msg.as_str() {
-			"quarantined" => quarantine(req, sub),
-			"private" => error(req, format!("r/{} is a private community", sub)).await,
-			"banned" => error(req, format!("r/{} has been banned from Reddit", sub)).await,
-			_ => error(req, msg).await,
-		},
 	}
 }
 
@@ -146,21 +172,28 @@ pub async fn add_quarantine_exception(req: Request<Body>) -> Result<Response<Bod
 
 pub fn can_access_quarantine(req: &Request<Body>, sub: &str) -> bool {
 	// Determine if the subreddit can be accessed
-	setting(&req, &format!("allow_quaran_{}", sub.to_lowercase())).parse().unwrap_or_default()
+	setting(req, &format!("allow_quaran_{}", sub.to_lowercase())).parse().unwrap_or_default()
 }
 
-// Sub or unsub by setting subscription cookie using response "Set-Cookie" header
-pub async fn subscriptions(req: Request<Body>) -> Result<Response<Body>, String> {
+// Sub, filter, unfilter, or unsub by setting subscription cookie using response "Set-Cookie" header
+pub async fn subscriptions_filters(req: Request<Body>) -> Result<Response<Body>, String> {
 	let sub = req.param("sub").unwrap_or_default();
+	let action: Vec<String> = req.uri().path().split('/').map(String::from).collect();
+
 	// Handle random subreddits
 	if sub == "random" || sub == "randnsfw" {
-		return Err("Can't subscribe to random subreddit!".to_string());
+		if action.contains(&"filter".to_string()) || action.contains(&"unfilter".to_string()) {
+			return Err("Can't filter random subreddit!".to_string());
+		} else {
+			return Err("Can't subscribe to random subreddit!".to_string());
+		}
 	}
 
 	let query = req.uri().query().unwrap_or_default().to_string();
-	let action: Vec<String> = req.uri().path().split('/').map(String::from).collect();
 
-	let mut sub_list = Preferences::new(req).subscriptions;
+	let preferences = Preferences::new(req);
+	let mut sub_list = preferences.subscriptions;
+	let mut filters = preferences.filters;
 
 	// Retrieve list of posts for these subreddits to extract display names
 	let posts = json(format!("/r/{}/hot.json?raw_json=1", sub), true).await?;
@@ -181,8 +214,10 @@ pub async fn subscriptions(req: Request<Body>) -> Result<Response<Body>, String>
 	for part in sub.split('+') {
 		// Retrieve display name for the subreddit
 		let display;
-		let part = if let Some(&(_, display)) = display_lookup.iter().find(|x| x.0 == part.to_lowercase()) {
-			// This is already known, doesn't require seperate request
+		let part = if part.starts_with("u_") {
+			part
+		} else if let Some(&(_, display)) = display_lookup.iter().find(|x| x.0 == part.to_lowercase()) {
+			// This is already known, doesn't require separate request
 			display
 		} else {
 			// This subreddit display name isn't known, retrieve it
@@ -195,16 +230,28 @@ pub async fn subscriptions(req: Request<Body>) -> Result<Response<Body>, String>
 		if action.contains(&"subscribe".to_string()) && !sub_list.contains(&part.to_owned()) {
 			// Add each sub name to the subscribed list
 			sub_list.push(part.to_owned());
-			// Reorder sub names alphabettically
-			sub_list.sort_by_key(|a| a.to_lowercase())
+			filters.retain(|s| s.to_lowercase() != part.to_lowercase());
+			// Reorder sub names alphabetically
+			sub_list.sort_by_key(|a| a.to_lowercase());
+			filters.sort_by_key(|a| a.to_lowercase());
 		} else if action.contains(&"unsubscribe".to_string()) {
 			// Remove sub name from subscribed list
 			sub_list.retain(|s| s.to_lowercase() != part.to_lowercase());
+		} else if action.contains(&"filter".to_string()) && !filters.contains(&part.to_owned()) {
+			// Add each sub name to the filtered list
+			filters.push(part.to_owned());
+			sub_list.retain(|s| s.to_lowercase() != part.to_lowercase());
+			// Reorder sub names alphabetically
+			filters.sort_by_key(|a| a.to_lowercase());
+			sub_list.sort_by_key(|a| a.to_lowercase());
+		} else if action.contains(&"unfilter".to_string()) {
+			// Remove sub name from filtered list
+			filters.retain(|s| s.to_lowercase() != part.to_lowercase());
 		}
 	}
 
 	// Redirect back to subreddit
-	// check for redirect parameter if unsubscribing from outside sidebar
+	// check for redirect parameter if unsubscribing/unfiltering from outside sidebar
 	let path = if let Some(redirect_path) = param(&format!("?{}", query), "redirect") {
 		format!("/{}/", redirect_path)
 	} else {
@@ -225,6 +272,17 @@ pub async fn subscriptions(req: Request<Body>) -> Result<Response<Body>, String>
 				.finish(),
 		);
 	}
+	if filters.is_empty() {
+		response.remove_cookie("filters".to_string());
+	} else {
+		response.insert_cookie(
+			Cookie::build("filters", filters.join("+"))
+				.path("/")
+				.http_only(true)
+				.expires(OffsetDateTime::now_utc() + Duration::weeks(52))
+				.finish(),
+		);
+	}
 
 	Ok(response)
 }
@@ -239,6 +297,7 @@ pub async fn wiki(req: Request<Body>) -> Result<Response<Body>, String> {
 
 	let page = req.param("page").unwrap_or_else(|| "index".to_string());
 	let path: String = format!("/r/{}/wiki/{}.json?raw_json=1", sub, page);
+	let url = req.uri().to_string();
 
 	match json(path, quarantined).await {
 		Ok(response) => template(WikiTemplate {
@@ -246,6 +305,7 @@ pub async fn wiki(req: Request<Body>) -> Result<Response<Body>, String> {
 			wiki: rewrite_urls(response["data"]["content_html"].as_str().unwrap_or("<h3>Wiki not found</h3>")),
 			page,
 			prefs: Preferences::new(req),
+			url,
 		}),
 		Err(msg) => {
 			if msg == "quarantined" {
@@ -268,6 +328,7 @@ pub async fn sidebar(req: Request<Body>) -> Result<Response<Body>, String> {
 
 	// Build the Reddit JSON API url
 	let path: String = format!("/r/{}/about.json?raw_json=1", sub);
+	let url = req.uri().to_string();
 
 	// Send a request to the url
 	match json(path, quarantined).await {
@@ -282,6 +343,7 @@ pub async fn sidebar(req: Request<Body>) -> Result<Response<Body>, String> {
 			sub,
 			page: "Sidebar".to_string(),
 			prefs: Preferences::new(req),
+			url,
 		}),
 		Err(msg) => {
 			if msg == "quarantined" {
